require(timereg)
require(survival)
dat<-read.table("dat_hcvhbvhcc_survmed2.txt", h=T)
source("mediation_aalen_cox_ci_pval.R")
qq<-quantile(dat$logc[dat$logc>0])
dd<-qq["50%"]-qq["25%"]
dat$logc<-dat$logc/dd

method="Aalen"

## 1-mediator

ols_m<-glm(logb2~agegp2+agegp3+agegp4+GENDER+alt1+alt2+smoke1+alcohol1+logc, data=dat)
aalen_m2<-aalen(Surv(hcc.time/365.25, hcc.case)~const(agegp2)+const(agegp3)+const(agegp4)+
	const(GENDER)+const(alt1)+const(alt2)+const(smoke1)+const(alcohol1)+
	const(logc)+const(logb2), data=dat, robust=T)
cox_m2<-coxph(Surv(hcc.time/365.25, hcc.case)~agegp2+agegp3+agegp4+GENDER+
	alt1+alt2+smoke1+alcohol1+logc+logb2, data=dat)

if (method=="Aalen"){
	lambdas<-aalen_m2$gamma
	lambdas.var<-aalen_m2$robvar.gamma
} else if(method=="Cox"){
	lambdas<-cox_m2$coef
	lambdas.var<-cox_m2$var
}
mediation_ci1(lambdas[9], lambdas[10], lambdas.var[9,9], lambdas.var[9,10], lambdas.var[10,10],
	ols_m$coef[10], summary(ols_m)$cov.scaled[10,10], G=10^6, method=method)


## 2-mediator

ols_m1<-glm(logb1~agegp2+agegp3+agegp4+GENDER+alt1+alt2+smoke1+alcohol1+logc, data=dat)
ols_m2<-glm(logb2~agegp2+agegp3+agegp4+GENDER+alt1+alt2+smoke1+alcohol1+logc+logb1, data=dat)
aalen_m3<-aalen(Surv(hcc.time/365.25, hcc.case)~const(agegp2)+const(agegp3)+const(agegp4)+
	const(GENDER)+const(alt1)+const(alt2)+const(smoke1)+const(alcohol1)+
	const(logc)+const(logb1)+const(logb2), data=dat, robust=T)
cox_m3<-coxph(Surv(hcc.time/365.25, hcc.case)~agegp2+agegp3+agegp4+GENDER+
	alt1+alt2+smoke1+alcohol1+logc+logb1+logb2, data=dat)

if (method=="Aalen"){
	lambdas<-aalen_m3$gamma
	Sigma.lambda<-aalen_m3$robvar.gamma[9:11, 9:11]
} else if (method=="Cox"){
	lambdas<-cox_m3$coef
	Sigma.lambda<-cox_m3$var[9:11, 9:11]
}
alphas<-ols_m2$coef
Sigma.alpha<-summary(ols_m2)$cov.scaled[10:11, 10:11]
deltas<-ols_m1$coef
Sigma.delta<-summary(ols_m1)$cov.scaled[10, 10]

mediation_ci2(lambdas[9], lambdas[10], lambdas[11], Sigma.lambda, 
	alphas[10], alphas[11], Sigma.alpha, deltas[10], Sigma.delta, G=10^6, method=method)

